@@ -297,6 +297,7 @@ and newstyle_state_machine = [
    * NEGOTIATING after OPT_STRUCTURED_REPLY or any failed OPT_GO.
    *)
   Group ("OPT_STARTTLS", newstyle_opt_starttls_state_machine);
+  Group ("OPT_EXTENDED_HEADERS", newstyle_opt_extended_headers_state_machine);
   Group ("OPT_STRUCTURED_REPLY", newstyle_opt_structured_reply_state_machine);
   Group ("OPT_META_CONTEXT", newstyle_opt_meta_context_state_machine);
   Group ("OPT_GO", newstyle_opt_go_state_machine);
@@ -441,6 +442,46 @@ and newstyle_opt_list_state_machine = [
   };
 ]
 
+(* Fixed newstyle NBD_OPT_EXTENDED_HEADERS option.
+ * Implementation: generator/states-newstyle-opt-extended-headers.c
+ *)
+and newstyle_opt_extended_headers_state_machine = [
+  State {
+    default_state with
+    name = "START";
+    comment = "Try to negotiate newstyle NBD_OPT_EXTENDED_HEADERS";
+    external_events = [];
+  };
+
+  State {
+    default_state with
+    name = "SEND";
+    comment = "Send newstyle NBD_OPT_EXTENDED_HEADERS negotiation request";
+    external_events = [ NotifyWrite, "" ];
+  };
+
+  State {
+    default_state with
+    name = "RECV_REPLY";
+    comment = "Receive newstyle NBD_OPT_EXTENDED_HEADERS option reply";
+    external_events = [ NotifyRead, "" ];
+  };
+
+  State {
+    default_state with
+    name = "RECV_REPLY_PAYLOAD";
+    comment = "Receive any newstyle NBD_OPT_EXTENDED_HEADERS reply payload";
+    external_events = [ NotifyRead, "" ];
+  };
+
+  State {
+    default_state with
+    name = "CHECK_REPLY";
+    comment = "Check newstyle NBD_OPT_EXTENDED_HEADERS option reply";
+    external_events = [];
+  };
+]
+
 (* Fixed newstyle NBD_OPT_STRUCTURED_REPLY option.
  * Implementation: generator/states-newstyle-opt-structured-reply.c
  *)
@@ -769,13 +810,20 @@ and reply_state_machine = [
 
   State {
     default_state with
-    name = "CHECK_SIMPLE_OR_STRUCTURED_REPLY";
-    comment = "Check if the reply is a simple or structured reply";
+    name = "CHECK_REPLY_MAGIC";
+    comment = "Check if the reply has expected magic";
+    external_events = [];
+  };
+
+  State {
+    default_state with
+    name = "RECV_STRUCTURED_REMAINING";
+    comment = "Receiving the remaining part of a structured reply header";
     external_events = [];
   };
 
   Group ("SIMPLE_REPLY", simple_reply_state_machine);
-  Group ("STRUCTURED_REPLY", structured_reply_state_machine);
+  Group ("CHUNK_REPLY", chunk_reply_state_machine);
 
   State {
     default_state with
@@ -804,91 +852,84 @@ and simple_reply_state_machine = [
   };
 ]
 
-(* Receiving a structured reply from the server.
- * Implementation: generator/states-reply-structured.c
+(* Receiving a structured reply payload from the server.
+ * Implementation: generator/states-reply-chunk.c
  *)
-and structured_reply_state_machine = [
+and chunk_reply_state_machine = [
   State {
     default_state with
     name = "START";
-    comment = "Prepare to receive the remaining part of a structured reply";
-    external_events = [];
-  };
-
-  State {
-    default_state with
-    name = "RECV_REMAINING";
-    comment = "Receiving the remaining part of a structured reply";
-    external_events = [];
-  };
-
-  State {
-    default_state with
-    name = "CHECK";
-    comment = "Parse a structured reply from the server";
+    comment = "Start parsing a chunk reply payload from the server";
     external_events = [];
   };
 
   State {
     default_state with
     name = "RECV_ERROR";
-    comment = "Receive a structured reply error header";
+    comment = "Receive a chunk reply error header";
     external_events = []
   };
 
   State {
     default_state with
     name = "RECV_ERROR_MESSAGE";
-    comment = "Receive a structured reply error message";
+    comment = "Receive a chunk reply error message";
     external_events = [];
   };
 
   State {
     default_state with
     name = "RECV_ERROR_TAIL";
-    comment = "Receive a structured reply error tail";
+    comment = "Receive a chunk reply error tail";
     external_events = [];
   };
 
   State {
     default_state with
     name = "RECV_OFFSET_DATA";
-    comment = "Receive a structured reply offset-data header";
+    comment = "Receive a chunk reply offset-data header";
     external_events = [];
   };
 
   State {
     default_state with
     name = "RECV_OFFSET_DATA_DATA";
-    comment = "Receive a structured reply offset-data block of data";
+    comment = "Receive a chunk reply offset-data block of data";
     external_events = [];
   };
 
   State {
     default_state with
     name = "RECV_OFFSET_HOLE";
-    comment = "Receive a structured reply offset-hole header";
+    comment = "Receive a chunk reply offset-hole header";
+    external_events = [];
+  };
+
+  State {
+    default_state with
+    name = "RECV_BS_HEADER";
+    comment = "Receive header of a chunk reply block-status payload";
     external_events = [];
   };
 
   State {
     default_state with
     name = "RECV_BS_ENTRIES";
-    comment = "Receive a structured reply block-status payload";
+    comment = "Receive entries array of chunk reply block-status payload";
     external_events = [];
   };
 
   State {
     default_state with
     name = "RESYNC";
-    comment = "Ignore payload of an unexpected structured reply";
+    comment = "Ignore payload of an unexpected chunk reply";
     external_events = [];
   };
 
   State {
     default_state with
     name = "FINISH";
-    comment = "Finish receiving a structured reply";
+    comment = "Finish receiving a chunk reply";
     external_events = [];
   };
 ]
