@@ -35,6 +35,7 @@
 #include "nbdinfo.h"
 
 static void show_boolean (const char *name, bool cond);
+static void show_size (const char *name, int64_t size);
 static int collect_context (void *opaque, const char *name);
 static char *get_content (struct nbd_handle *, int64_t size);
 
@@ -181,13 +182,9 @@ show_one_export (struct nbd_handle *nbd, const char *desc,
       show_boolean ("can_trim", can_trim);
     if (can_zero >= 0)
       show_boolean ("can_zero", can_zero);
-    if (block_minimum > 0)
-      fprintf (fp, "\t%s: %" PRId64 "\n", "block_size_minimum", block_minimum);
-    if (block_preferred > 0)
-      fprintf (fp, "\t%s: %" PRId64 "\n", "block_size_preferred",
-               block_preferred);
-    if (block_maximum > 0)
-      fprintf (fp, "\t%s: %" PRId64 "\n", "block_size_maximum", block_maximum);
+    show_size ("block_size_minimum", block_minimum);
+    show_size ("block_size_preferred", block_preferred);
+    show_size ("block_size_maximum", block_maximum);
   }
   else {
     if (first)
@@ -304,6 +301,21 @@ show_boolean (const char *name, bool cond)
   ansi_restore (fp);
 }
 
+/* Used for displaying sizes in non-JSON output. */
+void show_size (const char *name, int64_t size)
+{
+  char size_str[HUMAN_SIZE_LONGEST];
+  bool human_size_flag = false;
+
+  if (size > 0) {
+    human_size (size_str, size, &human_size_flag);
+    if (human_size_flag)
+      fprintf (fp, "\t%s: %" PRId64 " (%s)\n", name, size, size_str);
+    else
+      fprintf (fp, "\t%s: %" PRId64 "\n", name, size);
+  }
+}
+
 static int
 collect_context (void *opaque, const char *name)
 {
@@ -343,10 +355,20 @@ get_content (struct nbd_handle *nbd, int64_t size)
   if (!probe_content)
     return NULL;
 
-  if (nbd_aio_is_negotiating (nbd) &&
-      nbd_opt_go (nbd) == -1) {
-    fprintf (stderr, "%s: %s\n", progname, nbd_get_error ());
-    exit (EXIT_FAILURE);
+  if (nbd_aio_is_negotiating (nbd)) {
+    if (nbd_opt_go (nbd) == -1) {
+      fprintf (stderr, "%s: %s\n", progname, nbd_get_error ());
+      exit (EXIT_FAILURE);
+    }
+    /* nbd-server 3.25 only reports accurate size to NBD_OPT_GO and 0
+     * to NBD_OPT_INFO; this hack won't fix what we report for size,
+     * but improves what we report for contents.
+     */
+    if (size == 0) {
+      size = nbd_get_size (nbd);
+      if (size == -1)
+        size = 0;
+    }
   }
 
   /* Write the first part of the NBD export to a temporary file. */
@@ -376,8 +398,11 @@ get_content (struct nbd_handle *nbd, int64_t size)
   if (fp == NULL)
     goto out;
   r = getline (&ret, &len, fp);
-  if (r == -1)
+  if (r == -1) {
+    free(ret);
+    ret = NULL;
     goto out;
+  }
 
   /* Remove trailing \n. */
   if (r > 0 && ret[r-1] == '\n')
